package com.citrusengine.objects.platformer {

	import com.citrusengine.objects.PhysicsObject;

	import org.osflash.signals.Signal;

	import flash.events.TimerEvent;
	import flash.utils.Timer;

	/**
	 * A cannon is an object which fires missiles. A cannon is a static body so it extends Platform.
	 * Properties:
	 * fireRate : The frequency that missiles are fired.
	 * startingDirection : The direction that missiles are fired.
	 * openFire : Indicate if the cannon shoot at start or not.
	 * 
	 * Events:
	 * onGiveDamage - Dispatched when the missile explodes on a PhysicsObject. Passes one parameter:
	 * 				  The Object it exploded on (PhysicsObject)
	 */
	public class Cannon extends Platform {
		
		/**
		 * The frequency that missiles are fired.
		 */
		[Property(value="2000")]
		public var fireRate:Number = 2000;
		
		/**
		 * The direction that missiles are fired
		 */
		[Property(value="right")]
		public var startingDirection:String = "right";
		
		/**
		 * Indicate if the cannon shoot at start or not.
		 */
		[Property(value="true")]
		public var openFire:Boolean = true;

		[Property(value="20")]
		public var missileWidth:uint = 20;

		[Property(value="20")]
		public var missileHeight:uint = 20;

		[Property(value="2")]
		public var missileSpeed:Number = 2;

		[Property(value="0")]
		public var missileAngle:Number = 0;

		[Property(value="1000")]
		public var missileExplodeDuration:Number = 1000;

		[Property(value="1000")]
		public var missileFuseDuration:Number = 10000;

		[Property(value="", browse="true")]
		public var missileView:String = "";

		/**
		 * onGiveDamage - Dispatched when the missile explodes on a PhysicsObject. Passes one parameter:
		 * 				  The Object it exploded on (PhysicsObject)
		 */
		public var onGiveDamage:Signal;

		protected var _firing:Boolean = false;

		protected var _timer:Timer;

		public function Cannon(name:String, params:Object = null) {
			
			super(name, params);

			onGiveDamage = new Signal(PhysicsObject);
			
			if (openFire)
				startFire();
		}

		override public function destroy():void {

			onGiveDamage.removeAll();

			_timer.stop();
			_timer.removeEventListener(TimerEvent.TIMER, _fire);

			super.destroy();
		}

		override public function update(timeDelta:Number):void {

			super.update(timeDelta);

			_updateAnimation();
		}

		protected function _damage(missile:Missile, contact:PhysicsObject):void {

			if (contact != null) {
				onGiveDamage.dispatch(contact);
			}
		}

		public function startFire():void {

			_firing = true;

			_timer = new Timer(fireRate);
			_timer.addEventListener(TimerEvent.TIMER, _fire);
			_timer.start();
		}

		public function stopFire():void {

			_firing = false;

			_timer.stop();
			_timer.removeEventListener(TimerEvent.TIMER, _fire);
		}

		protected function _fire(tEvt:TimerEvent):void {

			var missile:Missile;

			if (startingDirection == "right") {
				missile = new Missile("Missile", {x:x + width, y:y, width:missileWidth, height:missileHeight, speed:missileSpeed, angle:missileAngle, explodeDuration:missileExplodeDuration, fuseDuration:missileFuseDuration, view:missileView});
			} else {
				missile = new Missile("Missile", {x:x - width, y:y, width:missileWidth, height:missileHeight, speed:-missileSpeed, angle:missileAngle, explodeDuration:missileExplodeDuration, fuseDuration:missileFuseDuration, view:missileView});
			}

			_ce.state.add(missile);
			missile.onExplode.addOnce(_damage);
		}

		protected function _updateAnimation():void {

			if (_firing) {
				_animation = "fire";
			} else {
				_animation = "normal";
			}
		}
	}
}